@@ -15,9 +15,9 @@
 //! # Example
 //!
 //! ```
-//! # use plotpy::{Plot, Surface};
+//! # use plotpy::{Plot, StrError, Surface};
 //! # use std::path::Path;
-//! # fn main() -> Result<(), &'static str> {
+//! # fn main() -> Result<(), StrError> {
 //! use russell_lab::generate3d;
 //! let mut surface = Surface::new();
 //! surface
@@ -47,6 +47,9 @@
 //! ![example_main.svg](https://raw.githubusercontent.com/cpmech/plotpy/main/figures/example_main.svg)
 //!
 
+/// Defines a type alias for the error type as a static string
+pub type StrError = &'static str;
+
 // modules ////////////////////////////////////////
 mod as_matrix;
 mod as_vector;
@@ -61,6 +64,7 @@ mod plot;
 mod shapes;
 mod slope_icon;
 mod surface;
+mod surface_geometry;
 mod text;
 pub use crate::as_matrix::*;
 pub use crate::as_vector::*;
@@ -75,6 +79,7 @@ pub use crate::plot::*;
 pub use crate::shapes::*;
 pub use crate::slope_icon::*;
 pub use crate::surface::*;
+pub use crate::surface_geometry::*;
 pub use crate::text::*;
 
 // run code from README file
