@@ -1,4 +1,4 @@
-use plotpy::{Contour, Plot};
+use plotpy::{Contour, Plot, StrError};
 use russell_lab::generate3d;
 use std::fs::File;
 use std::io::{BufRead, BufReader};
@@ -7,7 +7,7 @@ use std::path::Path;
 const OUT_DIR: &str = "/tmp/plotpy/integ_tests";
 
 #[test]
-fn test_contour() -> Result<(), &'static str> {
+fn test_contour() -> Result<(), StrError> {
     // contour object and options
     let mut contour = Contour::new();
     contour
@@ -38,12 +38,12 @@ fn test_contour() -> Result<(), &'static str> {
     let file = File::open(path).map_err(|_| "cannot open file")?;
     let buffered = BufReader::new(file);
     let lines_iter = buffered.lines();
-    assert!(lines_iter.count() > 1500);
+    assert!(lines_iter.count() > 1470);
     Ok(())
 }
 
 #[test]
-fn test_contour_colors() -> Result<(), &'static str> {
+fn test_contour_colors() -> Result<(), StrError> {
     // contour object and options
     let mut contour = Contour::new();
     contour
@@ -65,10 +65,7 @@ fn test_contour_colors() -> Result<(), &'static str> {
 
     // save figure
     let path = Path::new(OUT_DIR).join("integ_contour_colors.svg");
-    match plot.save(&path) {
-        Err(_) => plot.print_log_file(&path)?,
-        Ok(_) => (),
-    }
+    plot.save(&path)?;
 
     // check number of lines
     let file = File::open(path).map_err(|_| "cannot open file")?;
@@ -79,7 +76,7 @@ fn test_contour_colors() -> Result<(), &'static str> {
 }
 
 #[test]
-fn test_contour_colormap_index() -> Result<(), &'static str> {
+fn test_contour_colormap_index() -> Result<(), StrError> {
     for index in 0..10 {
         // contour object and options
         let mut contour = Contour::new();
@@ -114,7 +111,7 @@ fn test_contour_colormap_index() -> Result<(), &'static str> {
 }
 
 #[test]
-fn test_contour_colormap_name() -> Result<(), &'static str> {
+fn test_contour_colormap_name() -> Result<(), StrError> {
     for name in ["Pastel1", "tab20c", "gnuplot2"] {
         // contour object and options
         let mut contour = Contour::new();
